use crate::dom::element::datatype::DataType;
use std::collections::HashMap;

pub mod datatype;

#[derive(Debug, Clone)]
pub struct Element {
    element_type: String,
    attributes: HashMap<String, String>,
    children: Vec<DataType>,
    pre_text: String,
}

#[macro_export]
macro_rules! element {
    ($el:literal$(, $($content: expr),*)?) => {
        Element::new($el)
        $($(.append_child($content))*)?
    };
}

impl Element {
    pub fn new<S: Into<String>>(t: S) -> Self {
        Element {
            element_type: t.into(),
            attributes: HashMap::new(),
            children: vec![],
            pre_text: String::new()
        }
    }

    pub fn set_pre_text<S: Into<String>>(&mut self, str: S) -> &mut Self {
        self.pre_text = str.into();
        self
    }

    pub fn append_child<C: Into<DataType>>(&mut self, child: C) -> &mut Self {
        self.children.push(child.into());
        self
    }

    pub fn set_attribute<K: Into<String>, V: Into<String>>(
        &mut self,
        key: K,
        value: V,
    ) -> &mut Self {
        self.attributes.insert(key.into(), value.into());
        self
    }

    pub fn get_attribute<S: Into<String>>(&self, attribute: S) -> Option<String> {
        self.attributes
            .get(attribute.into().as_str())
            .map(|s| s.clone())
    }

    pub fn set_id<S: Into<String>>(&mut self, id: S) -> &mut Self {
        self.set_attribute("id", id);
        self
    }

    pub fn add_class<S: Into<String>>(&mut self, class: S) -> &mut Self {
        let classes = match self.get_attribute("class") {
            Some(s) => s + format!(" {}", class.into()).as_str(),
            None => class.into(),
        };
        self.set_attribute("class", classes);
        self
    }

    pub fn set_string<S: Into<String>>(&mut self, str: S) {
        self.children = vec![str.into().into()];
    }

    pub fn get_element_by_id<S: Into<String>>(&mut self, needle: S) -> Option<&mut Element> {
        let needle = needle.into();
        let mut el = None;
        let elements: Vec<&mut Element> = self
            .children
            .iter_mut()
            .filter_map(|d| match d {
                DataType::Element(e) => Some(e),
                _ => None,
            })
            .collect();
        for e in elements {
            if e.get_attribute("id").unwrap_or(String::new()) == needle.clone() {
                return Some(e);
            } else {
                el = e.get_element_by_id(needle.clone());
            }
        }
        el
    }
}

#[cfg(feature = "actix")]
use actix_web::{HttpRequest, HttpResponse, Responder, body::BoxBody, http::{StatusCode, header::ContentType}};

#[cfg(feature = "actix")]
impl Responder for Element {
    type Body = BoxBody;

    fn respond_to(self, _: &HttpRequest) -> HttpResponse<Self::Body> {
        HttpResponse::build(StatusCode::OK)
            .insert_header(ContentType::html())
            .body(self.to_string())
    }
}

impl ToString for Element {
    fn to_string(&self) -> String {
        let mut s = String::new();
        s.push_str(self.pre_text.as_str());
        s.push_str(format!("<{}", self.element_type).as_str());
        for (k, v) in &self.attributes {
            s.push_str(format!(" {}=\"{}\"", k, v).as_str());
        }
        s.push('>');
        for c in &self.children {
            s.push_str(c.to_string().as_str());
        }
        s.push_str(format!("</{}>", self.element_type).as_str());
        s
    }
}
