
use crate::dom::element::Element;

pub enum Asset {
    JS { file: String, defer: bool },
    CSS { file: String },
}

pub struct Head {
    title: String,
    assets: Vec<Asset>,
    children: Vec<Element>
}

impl Head {
    pub fn set_title<S: Into<String>>(&mut self, title: S) -> &mut Self {
        self.title = title.into();
        self
    }

    pub fn add_js<S: Into<String>>(&mut self, file: S, defer: bool) -> &mut Self {
        self.assets.push(Asset::JS {
            file: file.into(),
            defer,
        });
        self
    }

    pub fn add_css<S: Into<String>>(&mut self, file: S) -> &mut Self {
        self.assets.push(Asset::CSS { file: file.into() });
        self
    }

    pub fn add_element(&mut self, e: Element) -> &mut Self {
        self.children.push(e);
        self
    }
}

impl Default for Head {
    fn default() -> Self {
        Head {
            title: String::default(),
            assets: vec![],
            children: vec![]
        }
    }
}

impl Into<Element> for Head {
    fn into(self) -> Element {
        let mut r = Element::new("head");
        self.children
            .into_iter()
            .for_each(|e| {
                r.append_child(e);
            });
        r.append_child(Element::new("title").append_child(self.title));
        for a in self.assets {
            match a {
                Asset::JS { file, defer } => r.append_child(
                    Element::new("script")
                        .set_attribute("src", file)
                        .set_attribute("defer", if defer { "true" } else { "false" }),
                ),
                Asset::CSS { file } => r.append_child(
                    Element::new("link")
                        .set_attribute("rel", "stylesheet")
                        .set_attribute("href", file),
                ),
            };
        }
        r
    }
}
